@@ -3,7 +3,7 @@
 //  Eidos
 //
 //  Created by Ben Haller on 4/6/15; split from eidos_functions.cpp 09/26/2022
-//  Copyright (c) 2015-2023 Philipp Messer.  All rights reserved.
+//  Copyright (c) 2015-2024 Philipp Messer.  All rights reserved.
 //	A product of the Messer Lab, http://messerlab.org/slim/
 //
 
@@ -47,14 +47,14 @@ EidosValue_SP Eidos_ExecuteFunction_grep(const std::vector<EidosValue_SP> &p_arg
 	EidosValue_Logical *invert_value = (EidosValue_Logical *)(p_arguments[6].get());
 	
 	// Figure out our parameters
-	const std::string &pattern = pattern_value->StringRefAtIndex(0, nullptr);
+	const std::string &pattern = pattern_value->StringRefAtIndex_NOCAST(0, nullptr);
 	size_t pattern_length = pattern.length();
 	int x_count = x_value->Count();
-	bool ignoreCase = ignoreCase_value->LogicalAtIndex(0, nullptr);
-	const std::string &grammar = grammar_value->StringRefAtIndex(0, nullptr);
-	const std::string &value = value_value->StringRefAtIndex(0, nullptr);
-	bool fixed = fixed_value->LogicalAtIndex(0, nullptr);
-	bool invert = invert_value->LogicalAtIndex(0, nullptr);
+	bool ignoreCase = ignoreCase_value->LogicalAtIndex_NOCAST(0, nullptr);
+	const std::string &grammar = grammar_value->StringRefAtIndex_NOCAST(0, nullptr);
+	const std::string &value = value_value->StringRefAtIndex_NOCAST(0, nullptr);
+	bool fixed = fixed_value->LogicalAtIndex_NOCAST(0, nullptr);
+	bool invert = invert_value->LogicalAtIndex_NOCAST(0, nullptr);
 	
 	if (pattern_length == 0)
 		EIDOS_TERMINATION << "ERROR (Eidos_ExecuteFunction_grep): function grep() requires pattern to be of length >= 1." << EidosTerminate(nullptr);
@@ -92,17 +92,17 @@ EidosValue_SP Eidos_ExecuteFunction_grep(const std::vector<EidosValue_SP> &p_arg
 	// Make our return value
 	EidosValue_SP result_SP(nullptr);
 	EidosValue_Logical *result_logical = nullptr;
-	EidosValue_Int_vector *result_int = nullptr;
-	EidosValue_String_vector *result_string = nullptr;
+	EidosValue_Int *result_int = nullptr;
+	EidosValue_String *result_string = nullptr;
 	
 	if (value_enum == kIndices)
 	{
-		result_int = (new (gEidosValuePool->AllocateChunk()) EidosValue_Int_vector());
+		result_int = (new (gEidosValuePool->AllocateChunk()) EidosValue_Int());
 		result_SP = EidosValue_SP(result_int);
 	}
 	else if ((value_enum == kElements) || (value_enum == kMatches))
 	{
-		result_string = (new (gEidosValuePool->AllocateChunk()) EidosValue_String_vector());
+		result_string = (new (gEidosValuePool->AllocateChunk()) EidosValue_String());
 		result_SP = EidosValue_SP(result_string);
 	}
 	else if (value_enum == kLogical)
@@ -117,7 +117,7 @@ EidosValue_SP Eidos_ExecuteFunction_grep(const std::vector<EidosValue_SP> &p_arg
 		// pattern is a fixed string, so use basic C++ string searching, honoring ignoreCase and invert
 		for (int i = 0; i < x_count; ++i)
 		{
-			const std::string &x_element = x_value->StringRefAtIndex(i, nullptr);
+			const std::string &x_element = x_value->StringRefAtIndex_NOCAST(i, nullptr);
 			size_t match_pos = std::string::npos;	// not valid when invert==T, which is why "match" is disallowed then
 			bool is_match = false;
 			
@@ -181,7 +181,7 @@ EidosValue_SP Eidos_ExecuteFunction_grep(const std::vector<EidosValue_SP> &p_arg
 		
 		for (int i = 0; i < x_count; ++i)
 		{
-			const std::string &x_element = x_value->StringRefAtIndex(i, nullptr);
+			const std::string &x_element = x_value->StringRefAtIndex_NOCAST(i, nullptr);
 			std::smatch match_info;
 			bool is_match = std::regex_search(x_element, match_info, pattern_regex);
 			
@@ -227,21 +227,13 @@ EidosValue_SP Eidos_ExecuteFunction_nchar(const std::vector<EidosValue_SP> &p_ar
 	
 	EidosValue_String *x_value = (EidosValue_String *)(p_arguments[0].get());
 	int x_count = x_value->Count();
+	const std::string *string_vec = x_value->StringData();
 	
-	if (x_count == 1)
-	{
-		result_SP = EidosValue_SP(new (gEidosValuePool->AllocateChunk()) EidosValue_Int_singleton(x_value->StringRefAtIndex(0, nullptr).size()));
-	}
-	else
-	{
-		const std::vector<std::string> &string_vec = *x_value->StringVector();
-		
-		EidosValue_Int_vector *int_result = (new (gEidosValuePool->AllocateChunk()) EidosValue_Int_vector())->resize_no_initialize(x_count);
-		result_SP = EidosValue_SP(int_result);
-		
-		for (int value_index = 0; value_index < x_count; ++value_index)
-			int_result->set_int_no_check(string_vec[value_index].size(), value_index);
-	}
+	EidosValue_Int *int_result = (new (gEidosValuePool->AllocateChunk()) EidosValue_Int())->resize_no_initialize(x_count);
+	result_SP = EidosValue_SP(int_result);
+	
+	for (int value_index = 0; value_index < x_count; ++value_index)
+		int_result->set_int_no_check(string_vec[value_index].size(), value_index);
 	
 	result_SP->CopyDimensionsFromValue(x_value);
 	
@@ -258,27 +250,24 @@ EidosValue_SP Eidos_ExecuteFunction_strcontains(const std::vector<EidosValue_SP>
 	EidosValue_Int *pos_value = (EidosValue_Int *)(p_arguments[2].get());
 	
 	int x_count = x_value->Count();
-	const std::string &s = s_value->StringRefAtIndex(0, nullptr);
-	int64_t pos = pos_value->IntAtIndex(0, nullptr);
+	const std::string &s = s_value->StringRefAtIndex_NOCAST(0, nullptr);
+	int64_t pos = pos_value->IntAtIndex_NOCAST(0, nullptr);
 	
 	if (s.length() == 0)
 		EIDOS_TERMINATION << "ERROR (Eidos_ExecuteFunction_strcontains): function strcontains() requires s to be of length >= 1." << EidosTerminate(nullptr);
 	if (pos < 0)
 		EIDOS_TERMINATION << "ERROR (Eidos_ExecuteFunction_strcontains): function strcontains() requires pos to be >= 0." << EidosTerminate(nullptr);
 	
-	if (x_count == 1)
+	if ((x_count == 1) && (x_value ->DimensionCount() == 1))
 	{
-		const std::string &x = x_value->StringRefAtIndex(0, nullptr);
+		const std::string &x = x_value->StringRefAtIndex_NOCAST(0, nullptr);
 		size_t index = x.find(s, pos);
 		
-		if (x_value ->DimensionCount() == 1)
-			result_SP = (index != std::string::npos ? gStaticEidosValue_LogicalT : gStaticEidosValue_LogicalF);
-		else
-			result_SP = EidosValue_Logical_SP(new (gEidosValuePool->AllocateChunk()) EidosValue_Logical{index != std::string::npos});
+		return (index != std::string::npos ? gStaticEidosValue_LogicalT : gStaticEidosValue_LogicalF);
 	}
 	else
 	{
-		const std::vector<std::string> &string_vec = *x_value->StringVector();
+		const std::string *string_vec = x_value->StringData();
 		
 		EidosValue_Logical *logical_result = (new (gEidosValuePool->AllocateChunk()) EidosValue_Logical())->resize_no_initialize(x_count);
 		result_SP = EidosValue_SP(logical_result);
@@ -305,32 +294,22 @@ EidosValue_SP Eidos_ExecuteFunction_strfind(const std::vector<EidosValue_SP> &p_
 	EidosValue_Int *pos_value = (EidosValue_Int *)(p_arguments[2].get());
 	
 	int x_count = x_value->Count();
-	const std::string &s = s_value->StringRefAtIndex(0, nullptr);
-	int64_t pos = pos_value->IntAtIndex(0, nullptr);
+	const std::string &s = s_value->StringRefAtIndex_NOCAST(0, nullptr);
+	int64_t pos = pos_value->IntAtIndex_NOCAST(0, nullptr);
 	
 	if (s.length() == 0)
 		EIDOS_TERMINATION << "ERROR (Eidos_ExecuteFunction_strfind): function strfind() requires s to be of length >= 1." << EidosTerminate(nullptr);
 	if (pos < 0)
 		EIDOS_TERMINATION << "ERROR (Eidos_ExecuteFunction_strfind): function strfind() requires pos to be >= 0." << EidosTerminate(nullptr);
 	
-	if (x_count == 1)
+	const std::string *string_vec = x_value->StringData();
+	EidosValue_Int *int_result = (new (gEidosValuePool->AllocateChunk()) EidosValue_Int())->resize_no_initialize(x_count);
+	result_SP = EidosValue_SP(int_result);
+	
+	for (int value_index = 0; value_index < x_count; ++value_index)
 	{
-		const std::string &x = x_value->StringRefAtIndex(0, nullptr);
-		size_t index = x.find(s, pos);
-		result_SP = EidosValue_SP(new (gEidosValuePool->AllocateChunk()) EidosValue_Int_singleton(index == std::string::npos ? -1 : (int64_t)index));
-	}
-	else
-	{
-		const std::vector<std::string> &string_vec = *x_value->StringVector();
-		
-		EidosValue_Int_vector *int_result = (new (gEidosValuePool->AllocateChunk()) EidosValue_Int_vector())->resize_no_initialize(x_count);
-		result_SP = EidosValue_SP(int_result);
-		
-		for (int value_index = 0; value_index < x_count; ++value_index)
-		{
-			size_t index = string_vec[value_index].find(s, pos);
-			int_result->set_int_no_check(index == std::string::npos ? -1 : (int64_t)index, value_index);
-		}
+		size_t index = string_vec[value_index].find(s, pos);
+		int_result->set_int_no_check(index == std::string::npos ? -1 : (int64_t)index, value_index);
 	}
 	
 	result_SP->CopyDimensionsFromValue(x_value);
@@ -347,24 +326,21 @@ EidosValue_SP Eidos_ExecuteFunction_strprefix(const std::vector<EidosValue_SP> &
 	EidosValue_String *s_value = (EidosValue_String *)(p_arguments[1].get());
 	
 	int x_count = x_value->Count();
-	const std::string &s = s_value->StringRefAtIndex(0, nullptr);
+	const std::string &s = s_value->StringRefAtIndex_NOCAST(0, nullptr);
 	
 	if (s.length() == 0)
 		EIDOS_TERMINATION << "ERROR (Eidos_ExecuteFunction_strprefix): function strprefix() requires s to be of length >= 1." << EidosTerminate(nullptr);
 	
-	if (x_count == 1)
+	if ((x_count == 1) && (x_value ->DimensionCount() == 1))
 	{
-		const std::string &x = x_value->StringRefAtIndex(0, nullptr);
+		const std::string &x = x_value->StringData()[0];
 		bool has_prefix = Eidos_string_hasPrefix(x, s);
 		
-		if (x_value ->DimensionCount() == 1)
-			result_SP = (has_prefix ? gStaticEidosValue_LogicalT : gStaticEidosValue_LogicalF);
-		else
-			result_SP = EidosValue_Logical_SP(new (gEidosValuePool->AllocateChunk()) EidosValue_Logical{has_prefix});
+		return (has_prefix ? gStaticEidosValue_LogicalT : gStaticEidosValue_LogicalF);
 	}
 	else
 	{
-		const std::vector<std::string> &string_vec = *x_value->StringVector();
+		const std::string *string_vec = x_value->StringData();
 		
 		EidosValue_Logical *logical_result = (new (gEidosValuePool->AllocateChunk()) EidosValue_Logical())->resize_no_initialize(x_count);
 		result_SP = EidosValue_SP(logical_result);
@@ -390,11 +366,11 @@ EidosValue_SP Eidos_ExecuteFunction_strsplit(const std::vector<EidosValue_SP> &p
 	
 	EidosValue_String *x_value = (EidosValue_String *)p_arguments[0].get();
 	EidosValue_String *sep_value = (EidosValue_String *)p_arguments[1].get();
-	EidosValue_String_vector *string_result = new (gEidosValuePool->AllocateChunk()) EidosValue_String_vector();
+	EidosValue_String *string_result = new (gEidosValuePool->AllocateChunk()) EidosValue_String();
 	result_SP = EidosValue_SP(string_result);
 	
-	const std::string &joined_string = x_value->StringRefAtIndex(0, nullptr);
-	const std::string &separator = sep_value->StringAtIndex(0, nullptr);
+	const std::string &joined_string = x_value->StringRefAtIndex_NOCAST(0, nullptr);
+	const std::string &separator = sep_value->StringAtIndex_NOCAST(0, nullptr);
 	std::string::size_type start_idx = 0, sep_idx;
 	
 	if (separator.length() == 0)
@@ -435,24 +411,21 @@ EidosValue_SP Eidos_ExecuteFunction_strsuffix(const std::vector<EidosValue_SP> &
 	EidosValue_String *s_value = (EidosValue_String *)(p_arguments[1].get());
 	
 	int x_count = x_value->Count();
-	const std::string &s = s_value->StringRefAtIndex(0, nullptr);
+	const std::string &s = s_value->StringRefAtIndex_NOCAST(0, nullptr);
 	
 	if (s.length() == 0)
 		EIDOS_TERMINATION << "ERROR (Eidos_ExecuteFunction_strsuffix): function strsuffix() requires s to be of length >= 1." << EidosTerminate(nullptr);
 	
-	if (x_count == 1)
+	if ((x_count == 1) && (x_value ->DimensionCount() == 1))
 	{
-		const std::string &x = x_value->StringRefAtIndex(0, nullptr);
+		const std::string &x = x_value->StringData()[0];
 		bool has_prefix = Eidos_string_hasSuffix(x, s);
 		
-		if (x_value ->DimensionCount() == 1)
-			result_SP = (has_prefix ? gStaticEidosValue_LogicalT : gStaticEidosValue_LogicalF);
-		else
-			result_SP = EidosValue_Logical_SP(new (gEidosValuePool->AllocateChunk()) EidosValue_Logical{has_prefix});
+		return (has_prefix ? gStaticEidosValue_LogicalT : gStaticEidosValue_LogicalF);
 	}
 	else
 	{
-		const std::vector<std::string> &string_vec = *x_value->StringVector();
+		const std::string *string_vec = x_value->StringData();
 		
 		EidosValue_Logical *logical_result = (new (gEidosValuePool->AllocateChunk()) EidosValue_Logical())->resize_no_initialize(x_count);
 		result_SP = EidosValue_SP(logical_result);
@@ -480,111 +453,61 @@ EidosValue_SP Eidos_ExecuteFunction_substr(const std::vector<EidosValue_SP> &p_a
 	int x_count = x_value->Count();
 	EidosValue *arg_last = p_arguments[2].get();
 	EidosValueType arg_last_type = arg_last->Type();
+	const std::string *string_vec = x_value->StringData();
+	EidosValue *arg_first = p_arguments[1].get();
+	int arg_first_count = arg_first->Count();
+	bool first_singleton = (arg_first_count == 1);
 	
-	if (x_count == 1)
+	if (!first_singleton && (arg_first_count != x_count))
+		EIDOS_TERMINATION << "ERROR (Eidos_ExecuteFunction_substr): function substr() requires the size of first to be 1, or equal to the size of x." << EidosTerminate(nullptr);
+	
+	EidosValue_String *string_result = (new (gEidosValuePool->AllocateChunk()) EidosValue_String())->Reserve(x_count);
+	result_SP = EidosValue_SP(string_result);
+	
+	int64_t first0 = arg_first->IntAtIndex_NOCAST(0, nullptr);
+	
+	if (arg_last_type != EidosValueType::kValueNULL)
 	{
-		const std::string &string_value = x_value->StringRefAtIndex(0, nullptr);
-		int64_t len = (int64_t)string_value.size();
-		EidosValue *arg_first = p_arguments[1].get();
-		int arg_first_count = arg_first->Count();
+		// last supplied
+		int arg_last_count = arg_last->Count();
+		bool last_singleton = (arg_last_count == 1);
 		
-		if (arg_first_count != x_count)
-			EIDOS_TERMINATION << "ERROR (Eidos_ExecuteFunction_substr): function substr() requires the size of first to be 1, or equal to the size of x." << EidosTerminate(nullptr);
+		if (!last_singleton && (arg_last_count != x_count))
+			EIDOS_TERMINATION << "ERROR (Eidos_ExecuteFunction_substr): function substr() requires the size of last to be 1, or equal to the size of x." << EidosTerminate(nullptr);
 		
-		int64_t first0 = arg_first->IntAtIndex(0, nullptr);
+		int64_t last0 = arg_last->IntAtIndex_NOCAST(0, nullptr);
 		
-		if (arg_last_type != EidosValueType::kValueNULL)
+		for (int value_index = 0; value_index < x_count; ++value_index)
 		{
-			// last supplied
-			int arg_last_count = arg_last->Count();
-			
-			if (arg_last_count != x_count)
-				EIDOS_TERMINATION << "ERROR (Eidos_ExecuteFunction_substr): function substr() requires the size of last to be 1, or equal to the size of x." << EidosTerminate(nullptr);
-			
-			int64_t last0 = arg_last->IntAtIndex(0, nullptr);
-			
-			int64_t clamped_first = (int)first0;
-			int64_t clamped_last = (int)last0;
+			const std::string &str = string_vec[value_index];
+			int64_t len = (int64_t)str.size();
+			int64_t clamped_first = (first_singleton ? first0 : arg_first->IntAtIndex_NOCAST(value_index, nullptr));
+			int64_t clamped_last = (last_singleton ? last0 : arg_last->IntAtIndex_NOCAST(value_index, nullptr));
 			
 			if (clamped_first < 0) clamped_first = 0;
 			if (clamped_last >= len) clamped_last = (int)len - 1;
 			
 			if ((clamped_first >= len) || (clamped_last < 0) || (clamped_first > clamped_last))
-				result_SP = EidosValue_SP(new (gEidosValuePool->AllocateChunk()) EidosValue_String_singleton(gEidosStr_empty_string));
+				string_result->PushString(gEidosStr_empty_string);
 			else
-				result_SP = EidosValue_SP(new (gEidosValuePool->AllocateChunk()) EidosValue_String_singleton(string_value.substr(clamped_first, clamped_last - clamped_first + 1)));
-		}
-		else
-		{
-			// last not supplied; take substrings to the end of each string
-			int clamped_first = (int)first0;
-			
-			if (clamped_first < 0) clamped_first = 0;
-			
-			if (clamped_first >= len)						
-				result_SP = EidosValue_SP(new (gEidosValuePool->AllocateChunk()) EidosValue_String_singleton(gEidosStr_empty_string));
-			else
-				result_SP = EidosValue_SP(new (gEidosValuePool->AllocateChunk()) EidosValue_String_singleton(string_value.substr(clamped_first, len)));
+				string_result->PushString(str.substr(clamped_first, clamped_last - clamped_first + 1));
 		}
 	}
 	else
 	{
-		const std::vector<std::string> &string_vec = *x_value->StringVector();
-		EidosValue *arg_first = p_arguments[1].get();
-		int arg_first_count = arg_first->Count();
-		bool first_singleton = (arg_first_count == 1);
-		
-		if (!first_singleton && (arg_first_count != x_count))
-			EIDOS_TERMINATION << "ERROR (Eidos_ExecuteFunction_substr): function substr() requires the size of first to be 1, or equal to the size of x." << EidosTerminate(nullptr);
-		
-		EidosValue_String_vector *string_result = (new (gEidosValuePool->AllocateChunk()) EidosValue_String_vector())->Reserve(x_count);
-		result_SP = EidosValue_SP(string_result);
-		
-		int64_t first0 = arg_first->IntAtIndex(0, nullptr);
-		
-		if (arg_last_type != EidosValueType::kValueNULL)
+		// last not supplied; take substrings to the end of each string
+		for (int value_index = 0; value_index < x_count; ++value_index)
 		{
-			// last supplied
-			int arg_last_count = arg_last->Count();
-			bool last_singleton = (arg_last_count == 1);
+			const std::string &str = string_vec[value_index];
+			int64_t len = (int64_t)str.size();
+			int64_t clamped_first = (first_singleton ? first0 : arg_first->IntAtIndex_NOCAST(value_index, nullptr));
 			
-			if (!last_singleton && (arg_last_count != x_count))
-				EIDOS_TERMINATION << "ERROR (Eidos_ExecuteFunction_substr): function substr() requires the size of last to be 1, or equal to the size of x." << EidosTerminate(nullptr);
+			if (clamped_first < 0) clamped_first = 0;
 			
-			int64_t last0 = arg_last->IntAtIndex(0, nullptr);
-			
-			for (int value_index = 0; value_index < x_count; ++value_index)
-			{
-				const std::string &str = string_vec[value_index];
-				int64_t len = (int64_t)str.size();
-				int64_t clamped_first = (first_singleton ? first0 : arg_first->IntAtIndex(value_index, nullptr));
-				int64_t clamped_last = (last_singleton ? last0 : arg_last->IntAtIndex(value_index, nullptr));
-				
-				if (clamped_first < 0) clamped_first = 0;
-				if (clamped_last >= len) clamped_last = (int)len - 1;
-				
-				if ((clamped_first >= len) || (clamped_last < 0) || (clamped_first > clamped_last))
-					string_result->PushString(gEidosStr_empty_string);
-				else
-					string_result->PushString(str.substr(clamped_first, clamped_last - clamped_first + 1));
-			}
-		}
-		else
-		{
-			// last not supplied; take substrings to the end of each string
-			for (int value_index = 0; value_index < x_count; ++value_index)
-			{
-				const std::string &str = string_vec[value_index];
-				int64_t len = (int64_t)str.size();
-				int64_t clamped_first = (first_singleton ? first0 : arg_first->IntAtIndex(value_index, nullptr));
-				
-				if (clamped_first < 0) clamped_first = 0;
-				
-				if (clamped_first >= len)						
-					string_result->PushString(gEidosStr_empty_string);
-				else
-					string_result->PushString(str.substr(clamped_first, len));
-			}
+			if (clamped_first >= len)						
+				string_result->PushString(gEidosStr_empty_string);
+			else
+				string_result->PushString(str.substr(clamped_first, len));
 		}
 	}
 	
